function [Mix, tstop, Qs] = mixestims(Mix, frg, ndat, Mixa)
%  quasi-Bayes estimation of ARX mixture with stopping 
%  Mix = mixestims(Mix0, frg, ndat, Mixa)
%  Mix = mixestims(Mix0, frg, ndat)     % stabilized forgetting is not used
%  Mix = mixestims(Mix0, frg)           % recursive estimation
%  Mix = mixestims(Mix0)                % use default forgetting rate
%
% Mix     : estimated mixture
% Mix0    : initial mixture
% frg     : forgetting rate
% ndat    : sample size
% Mixa    : alternative mixture for stabilized forgetting
%
% Design  : P. Nedoma
% Updated : January 2003
% Project : ProDaCTools continuation

global TIME;
Mix = mix2mix(Mix, 21);

if nargin<3,  ndat = 1; end
if nargin<2,  frg = defaults('frg'); end

x = defaults('E');
threshold = x(1);  % 0.0025;           

% recursive call
if nargin<3  | ndat==1
   tstop = 0;
   Qs    = 0;
   Mix = mixupdt(Mix, 0); 
   ll = Mix.states.mixll;              % current value of mixll
   ll=exp(ll);

  if ~isfield(Mix.states, 'Fac')       % 	in C, check length
     Mix.states.Fac  = facarx(1, [ 0; 1]);     
  else
    tstop = 0;
    [Mix.states.Fac, Qs] = stopstac(Mix.states.Fac, [ll, 1] );
    if Qs<threshold,  tstop=TIME;  end
  end
   
   if frg<1,
      if nargin<4, Mix = mixfrg(Mix, frg);
      else         Mix = mixfrg(Mix, frg, frg, Mixa);
      end
   end

return
end

% stoping rules
Fac  = facarx(1, [ 0; 1]);             % auxiliary factor feeded by mixll

[ncom,nchn] = size(Mix.coms);
nfac = length(Mix.states.facs);

% state counters
comlls  = zeros(1, ncom);
mixll   = 0;
comwgs  = zeros(1, ncom);
facwgs  = zeros(1, nfac);
faceps  = zeros(1, nfac);

maxtd = Mix.states.maxtd;

for TIME = maxtd+1:ndat
% quasi-Bayes update
    Mix = mixupdt(Mix, 0);
%disp(Mix.dfcs);
% forgetting
   if frg<1,
      if nargin<4, Mix = mixfrg(Mix, frg);
      else         Mix = mixfrg(Mix, frg, frg, Mixa);
      end
   end

% states counters
comlls  = comlls + Mix.states.comlls;
mixll   = mixll  + Mix.states.mixll;
comwgs  = comwgs + Mix.states.comwgs;
facwgs  = facwgs + Mix.states.facwgs;
faceps  = faceps + Mix.states.faceps.*Mix.states.faceps;

% stoping rules
    ll = Mix.states.mixll; % current value of mixll
    ll=exp(ll);
    mixlls(TIME) = ll;
    [Fac,Q] = stopstac(Fac, [ll, 1] );
    Qs(TIME) = Q;
    if Q<threshold,  tstop = TIME; break; end
end
tstop = TIME;

% update states
Mix.states.comlls = comlls;
Mix.states.mixll  = mixll;
Mix.states.comwgs = comwgs;
Mix.states.facwgs = facwgs;
Mix.states.faceps = faceps;
