% zstopest.m
% STOPPING RULES IN ESTIMATION\n');
%
% Design : P. Nedoma
% Updated: December 2004
% Project: GACR 102/03/0049, AVCR S1075351, 1ET100750401, Edukalibre


prodini                                % startup routine
echo off
idef = [];

while 1 % ==============================================
if isempty(idef)
ndat = 400;                          % sample size       
ncom = 5;                            % sample size
dia  = 0.1;
seed = 123;                          % seed of random generator
threshold = 0.0025;
end

clc
fprintf('\nSTOPPING RULES IN ESTIMATION\n');
fprintf('--- system static, 2 channels ----------------\n');
fprintf('number of components         ->   ncom  = %i;\n',       ncom);
fprintf('diagonal of noise covariance ->   dia   = %g;\n',       dia);
fprintf('\n--- Measurements ---------------------------\n');
fprintf('length                       ->   ndat  = %i;\n',       ndat);
fprintf('seed                         ->   seed  = %i;\n',       seed);
fprintf('\n--- processing ------------------------------------\n');
fprintf('threshold                    ->   threshold = %g;\n',     threshold);
dial1;
                                       % size of data sample
cove = ltdl([dia 0.1*dia ;0.1*dia dia]);     % common component noise variance
if seed, randn('seed',seed);rand('seed',seed); end
Sim  = statsim(ndat, ncom, cove);      % build static mixture

sub = 220;
LegendSize = 10;

clf; figure(1);

sub=sub+1; subplot(sub);
[x,y,xy] = mixgrid(Sim); contour(x,y,xy,15);
H=legend('Simulator');
set(H, 'fontSize', LegendSize); 
grid on
xlabel('data 1'); ylabel('data 2');
drawnow

Mix0 = mix2mix(genmixe(ncom), 21);     % initial mixture


Mix = mixestim(Mix0, 1, ndat);
ep  = relep(Mix,1,1);

sub=sub+1; subplot(sub);
[x,y,xy] = mixgrid(Mix); contour(x,y,xy,15);
%ax = axis; ax(1)=0; ax(2)=KONEC; axis(ax);
grid on
xlabel('time');
H=legend('no stopping',['pred. error norm=',num2str(ep)]);
set(H, 'fontSize', LegendSize); 

Mix  = Mix0;
Fac  = facarx(1, [ 0; 1]);             % auxiliary factor feeded by mixll

echo off

%[Mix,tstop, Qs, mixlls] = mixestims(Mix, 1, ndat);

vll = 0;
for TIME=Mix.states.maxtd+1:ndat
    [Mix, tstop, Q] = mixestims(Mix);
    Qs(TIME)=Q;
    vll = vll + Mix.states.mixll;
%    if tstop, break; end
    if (TIME>20) & (Q< threshold), break; end  
end

KONEC=TIME;


ep  = relep(Mix,1,1);

sub=sub+1; subplot(sub);
[x,y,xy] = mixgrid(Mix); contour(x,y,xy,15);

grid on
xlabel('time');
H=legend('with stopping',['pred. error norm=',num2str(ep)],...
    [' TIME= ',int2str(KONEC)]);
set(H, 'fontSize', LegendSize); 


sub=sub+1; subplot(sub);
plot(Qs,':'); %hold on; plot(Qs,':')
ax = axis; ax(4)=0.3; axis(ax);

hold on
plot([1 KONEC], [threshold threshold], '--');
H=legend('Q', 'threshold');


set(H, 'fontSize', LegendSize); 
%plot([1 len], [0.01 0.01], ':');
%plot([1 len], [0.02 0.02], ':');
%plot([1 len], [0.03 0.03], ':'); 

ax = axis; ax(1)=0; ax(2)=KONEC; axis(ax);
title(['stopping time ', int2str(tstop)]);
xlabel('time');
drawnow

plt('zstopest_');
dial2;

end     



