% zestcmp.m
% COMPARISON OF MIXTURE ESTIMATION ALGORITHMS
%
% Design : P. Nedoma
% Updated: December 2002, MK October 2004
% Project: GACR 102/03/0049, AVCR S1075351, 1ET100750401, Edukalibre

prodini
echo off
idef = [];


while 1 % ==============================================
if isempty(idef)
ndat = 500;                          % sample size       
ncom = 5;                            % sample size
diac = 0.05;
frg  = 1;                            % forgetting rate
frga = 0.6;                          % alternative 
seed = 123;                          % seed of random generator
niter= 10;                           % No. of iterations
end

clc
fprintf('\nCOMPARISON OF MIXTURE ESTIMATION ALGORITHMS\n');
fprintf('\n--- system ------------------------------------\n');
fprintf('number of components         ->   ncom  = %i;\n',       ncom);
fprintf('diagonal of noise covariance ->   diac   = %g;\n',      diac);
fprintf('\n--- Measurements -----------------------------\n');
fprintf('length                       ->   ndat  = %i;\n',       ndat);
fprintf('seed                         ->   seed  = %i;\n',       seed);
fprintf('\n--- estimation --------------------------------\n');
fprintf('number of iterations   ->   niter = %i;\n',     niter);
fprintf('forgetting rate        ->   frg   = %g;\n',       frg);
fprintf('alternative forgetting ->   frga  = %g;\n',       frga);
fprintf('------------------------------------------------\n');

dial1;


cove = ltdl([diac, 0.1*diac; 0.1*diac diac]); % common noise variance
randn('seed', seed); 
Sim  = statsim(ndat, ncom, cove);    % create static mixture

% ----------------------------------------------------------
sub  = 330;
LegendSize=10;

clf
sub  = sub + 1; subplot(sub);
[x,y,z] = mixgrid(Sim); contour(x,y,z,15); grid on
H=legend('Simulator',0); 
set(H, 'fontSize', LegendSize); grid on
drawnow

Mix0 = genmixe(ncom);

method = 'p';                          % quai-Bayes estimation
Mix = mixest(Mix0, frg, ndat, niter, method); % estimation
err = relep(Mix);
Mixp = Mix;

sub  = sub + 1; subplot(sub);
[x,y,z] = mixgrid(Mix); contour(x,y,z,15); grid on
H=legend('PB',0); 
set(H, 'fontSize', LegendSize); grid on
drawnow

method = 'q';                          % quai-Bayes estimation
Mix = mixest(Mix0, frg, ndat, niter, method); % estimation
err    = [err, relep(Mix)];
Mixq = Mix;

sub  = sub + 1; subplot(sub);
[x,y,z] = mixgrid(Mix); contour(x,y,z,15); grid on
H=legend('QB',0); 
set(H, 'fontSize', LegendSize); grid on
drawnow

method = 'b';                          % batch quasi-Bayes
Mix = mixest(Mix0, frg, ndat, niter, method); % estimation
err    = [err, relep(Mix)];

sub  = sub + 1; subplot(sub);
[x,y,z] = mixgrid(Mix); contour(x,y,z,15); grid on
H=legend('BQB',0); 
set(H, 'fontSize', LegendSize); grid on
drawnow

method = 'f';                          % forgetting branching
Mix = mixest(Mix0, frga, ndat, niter, method); % estimation
err = [err, relep(Mix)];

sub  = sub + 1; subplot(sub);
[x,y,z] = mixgrid(Mix); contour(x,y,z,15); grid on
H=legend('BFRG',0); 
set(H, 'fontSize', LegendSize); grid on
drawnow

method = 'Q';                          % batch quasi-Bayes
%[Mix, tstop] = mixest(Mix0, frg, ndat, niter, method); % estimation
[Mix,tstop] = mixestqbs(Mix0, frg, ndat, niter);
err    = [err, relep(Mix)];

sub  = sub + 1; subplot(sub);
[x,y,z] = mixgrid(Mix); contour(x,y,z,15); grid on
H=legend('QX',0); 
set(H, 'fontSize', LegendSize); grid on
drawnow



method = 'P';                          % batch quasi-Bayes
%[Mix, tstop] = mixest(Mix0, frg, ndat, niter, method); % estimation
[Mix,tstop] = mixestimps(Mix0, frg, ndat, niter);
%%%[Mix, Mixq] = fixmixll(Mix, Mixp)
err = [err, relep(Mix)];

sub  = sub + 1; subplot(sub);
[x,y,z] = mixgrid(Mix); contour(x,y,z,15); grid on
H=legend('PX',0); 
set(H, 'fontSize', LegendSize); grid on
drawnow


sub  = sub + 1; subplot(sub);
plot(0,0,'ob', 0,0,'db', 0,0,'xb', 0,0,'*b', 0,0,'>b', 0,0,'<b');
H=legend('PB projection', 'QB quasi-Bayes','BQB batch QB',...
  'BFRG frg. branching', 'QX QB with stopping' , 'PX PB with stopping'); 
set(H, 'fontSize', LegendSize); grid on
drawnow


sub  = sub + 1; subplot(sub);
plot(1,err(1), 'o');  hold on
plot(2,err(2), 'd'); 
plot(3,err(3), 'x')
plot(4,err(4), '*');
plot(5,err(5), '>');
plot(6,err(6), '<');
plot(err,':');
ax = axis; ax(1)=0.5; ax(2)=6.5; axis(ax);
H=legend('pred. err. norm',0);
set(H, 'fontSize', LegendSize); grid on
%setaxis([231:234]);

plt('zestcmp_')

dial2
end % ====================================