% case study: stoping rules
%
% Design : P. Nedoma
% Updated: October 2003
% Project: GA CR 102/03/0049, AV CR S1075351, S1075102, DESIGNER
prodini                                % startup routine
echo off
idef = [];


while 1 % ==============================================
if isempty(idef)
ndat = 500;                          % sample size       
covu = 0.01^2;
covy = 0.01^2;
seed = 123;                          % seed of random generator
threshold = 0.0025;
end

clc
fprintf('Case study: stoping rules, factor estimation\n');
fprintf('--- system SISO, 2nd order, open loop ---------------\n');
fprintf('output noise covariance -> covu = %g\n',   covu);
fprintf('input noise covariance  -> covu = %g;\n',  covu);

fprintf('\n--- data ------------------------------------\n');
fprintf('size        ->   ndat  = %i;\n',       ndat);
fprintf('seed        ->   seed  = %i;\n',       seed);


fprintf('\n--- processing ------------------------------------\n');
fprintf('threshold   ->   threshold = %i;\n',     threshold);
fprintf('-----------------------------------------------\n');

dial1;
clf; figure(1)

stdy = sqrt(covy);
stdu = sqrt(covu);

% build factor
str  = [1 1 2 2; 1 2 0 1];
Eth  = [1.81 -0.8187  0.00468  0.00438];
Fac  = facarxls(1,str);              % build 1st ARX simulation factor
Fac.Eth  = Eth;
Fac.cove = stdy;

% generate data sample
if seed, randn('seed', seed); rand('seed',seed); end
DATA = [zeros(1, ndat); stdu*randn(1,ndat)];
Sim  = mixconst(Fac, 1, 1);
mixsimul(Sim, ndat);

% build initial factor
Fac  = facarx(1, str);
echo off
% recursive factor estimation
for TIME = 3:ndat
   [Fac, Q] = stopstac(Fac);           % one step of estimation
  % store result
   Qs(TIME)  = Q;
   F         = arx2arx(Fac);
   a1s(TIME) = F.Eth(1);
   a2s(TIME) = F.Eth(2);
   b1s(TIME) = F.Eth(3);
   b2s(TIME) = F.Eth(4);
end

sub = 120;
LegendSize = 14;



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
tr = find(Qs(10:end)<threshold);
tr = tr(1);


sub=sub+1; subplot(sub);
plot(a1s); hold on ; plot(a2s,'--' );
hold on
plot([1 ndat], [1.81 1.81], ':');
plot([1 ndat], [-0.8187 -0.8187], ':');

ax = axis; y = ax(3:4);
plot([tr tr], y, ':');
H=legend('a1','a2');
set(H, 'fontSize', LegendSize); 
xlabel('time');
drawnow


sub=sub+1; subplot(sub);
plot(Qs);
a=axis;
a(4) = 0.1; axis(a);
xlabel('time');
hold on
%plot([1 ndat], [0.01 0.01], ':');
%plot([1 ndat], [0.02 0.02], ':');
plot([1 ndat],  [threshold threshold], ':');


ax = axis; y = ax(3:4);
plot([tr tr], y, ':');
H=legend('Q','threshold');
set(H, 'fontSize', LegendSize); 
drawnow

%disp('EDIT'); pause; print -depsc zstop1
plt('zstop1');


dial2

end % ====================================
