% case study: stoping rules
%
% Design : P. Nedoma
% Updated: October 2003
% Project: GA CR 102/03/0049, AV CR S1075351, S1075102, DESIGNER
prodini                                % startup routine
echo off


idef = [];
while 1 % ==============================================
if isempty(idef)
ndat = 300;                          % sample size       
ncom = 3;                            % sample size
dia  = 0.1;
seed = 12;                           % seed of random generator
threshold = 0.0025;
end

clc
fprintf('\n Case study: stopping rules setup\n');
fprintf('--- system -------------------------------------\n');
fprintf('number of components   ->   ncom  = %i;\n',       ncom);
fprintf('diagonal of noise cov. ->   dia   = %g;\n',       dia);
fprintf('\n--- data sample--------------------------------\n');
fprintf('size                   ->   ndat  = %i;\n',       ndat);
fprintf('seed                   ->   seed  = %i;\n',       seed);
fprintf('\n--- processing---------------------------------\n');
fprintf('threshold              ->   threshold  = %i;\n',   threshold);
  
dial1;


clf; figure(1)                                     % size of data sample
cove = ltdl([dia 0.1*dia ;0.1*dia dia]);     % common component noise variance
if seed, randn('seed',seed);rand('seed',seed); end
Sim  = statsim(ndat, ncom, cove);      % build static mixture

Mix0 = mix2mix(genmixe(ncom), 21);     % initial mixture
Mix  = Mix0;

DATA = [DATA; zeros(1, ndat)];         % added data channel
Fac0 = facarx(3, [ 0; 1]);             % auxiliary factor feeded by mixll


beta = 0.7;
k=0;

for rep = 1:100  % ====================================
    mixsimul(Sim, ndat);
    Mix   = Mix0;
    Fac   = Fac0;
    mixll = 0;

for TIME  = 1:ndat % -------------------------------------
    Mix   = mixestimp(Mix, 1);           % step of mixture update
    mixll = Mix.states.mixll;
    [Fac,Q]  = stopstac(Fac, [mixll, 1] );
    if Q<threshold, break; end;
end
TIMEs(rep) = TIME;

[Cl,Cu,Chat,flag,Q]=credits(TIMEs,beta,threshold);
    Cls(rep) = Cl; Cus(rep) = Cu; Chats(rep)=Chat;
    Qs(rep)  = Q;
    if Q<threshold & k==0, k=rep; break;  end
end

sub = 220;
LegendSize = 14;


clf; figure(1)
sub=sub+1; subplot(sub);
plot(DATA(1,:),DATA(2,:),'.','MarkerSize',4);
hold on
statplot(Sim, '--');
H=legend('Data clusters', 'components');
set(H, 'fontSize', LegendSize); 
xlabel('1st channel'); ylabel('2nd channel');
grid on
drawnow


sub=sub+1; subplot(sub);
plot(TIMEs,'o');
hold on
plot(TIMEs,':');
H=legend('Stopping time');
set(H, 'fontSize', LegendSize); 
grid on
xlabel('realizations');
ylabel('time');
drawnow

rep                                    % number of repetions
[Cl, Chat, Cu]                         % final credibility interval

if 1
sub=sub+1; subplot(sub);
plot(Qs); hold on; %plot(Qs,':');
ax = axis; ax(4) = 0.1; axis(ax);
len = length(Qs);
hold on
%plot([k k],[0 0.1]);
plot([1 len], [threshold threshold], ':');
H=legend('Q', 'threshold');
set(H, 'fontSize', LegendSize); 
xlabel('realizations');
drawnow
end

if 1
sub=sub+1; subplot(sub);
plot(Chats,'-'); hold on;
plot(Cls  ,':');
plot(Cus  ,'-.');
H=legend('Chat', 'Cls', 'Cus');
set(H, 'fontSize', LegendSize); 
title('Estimation of credibility interval')
%plot(Chats, '-.'); plot(Cls  , '-.'); plot(Cus  , '-.');
ax = axis; ax(3)=50; axis(ax);
xlabel('time');
end
plt('zstop3');
dial2;
end