% zvermimo.m
% STOPPING RULES IN SIMULATION
%
% Design : P. Nedoma
% Updated: DECEMBER 2004
% Projects : GACR 102/03/0049, AVCR S1075351, 1ET100750401, Edukalibre
%


prodini                                % startup routine

echo off
idef = [];

while 1 % ==============================================
if isempty(idef)
threshold  = 0.005;
covy       = 1e-4;
covu       = 1e-4;
ndat       = 1000;
beta       = 0.7;    
seed       = 12345;
maxdat     = 400;
% for simeval
chns       = [1 3];                            % list of channels
maxrep     = 50;                               % maximum number of repetitions
end % ==================================================

clc
fprintf('STOPPING RULES IN SIMULATION\n');
fprintf('--------------------------------------------------------------- \n');
fprintf('\nSystems: SISO, dynamic factor 2nd order,  open loop\n');
fprintf('factor noise covariance ->   covy   = %g\n', covy);
fprintf('open loop covariance    ->   covu   = %g;\n',covu);

fprintf('\n---  Data for estimation -----------------------\n');
fprintf('size                    ->   ndat   = %i;\n',       ndat);
fprintf('seed                    ->   seed   = %i;\n',       seed);

fprintf('\n--- Stopping rules -----------------------------\n');
fprintf('maximum data length     ->   maxdat     = %i;\n',      maxdat);
fprintf('maximum repetitions     ->   maxrep     = %i;\n',      maxrep);
fprintf('threshold               ->   threshold  = %g;\n',      threshold);
fprintf('credibility level       ->   beta       = %g;\n',      beta);
fprintf('--------------------------------------------------\n');

dial1;

clf; figure(1);
if seed, randn('seed', seed); end
sub        = 120;
LegendSize = 12;

% ==========================================================
% simulation
% ==========================================================
% model for output: dynamic factor (model 1 from database of models)
str = [1 1 2 2; 1 2 0 1];              % factor structure
Eth = [1.81 -0.8187  0.00468 0.00438]; % regression coeficients
Fac = facarxls(1, str);                % output factor
Fac.Eth  = Eth;                        % factor regression coefficient
Fac.cove = covy;                       % noise covariance
Facs{1}  = Fac;

% 2nd output - channel 3, cove and Eth above
str = [3 3 4 4; 1 2 0 1];
Fac.ychn = 3;
Fac.str  = str;
Facs{2}  = Fac;

% simulator
Sim = mixconst(Facs, [1 2], 1);        % build simulator

% get data sample

stde = sqrt(covu);
DATA = [ zeros(1, ndat); randn(1,ndat)*stde
         zeros(1, ndat); randn(1,ndat)*stde];
mixsimul(Sim, ndat);                   % get data sample

% ==========================================================
% estimation of regression coeficients
% ==========================================================

% 1st output
str = [1 1 2 2; 1 2 0 1];              % factor structure
Fac = facarxls(1, str);                % output factor
Fac.dfm  = 10;
Fac.cove = 0.0001;
diagCth  = 100;
Fac.Cth  = diagCth*diag(ones(1,4));
Facs{1}  = Fac;

% second output
str = [3 3 4 4; 1 2 0 1];
Fac.ychn = 3;
Fac.str  = str;
Facs{2}  = Fac;
Mix0     = mixconst(Facs, [1 2], 1);
frg      = 1;
Mix      = mixestim(Mix0, frg, ndat);      % batch mixture estimation

disp('estimation results');
prt('true coefficients',Eth);
Fac = arx2arx( Mix.Facs{1} );
prt('estimated, 1st channel', Fac.Eth);
Fac = arx2arx( Mix.Facs{2} );
prt('estimated, 2nd channel',Fac.Eth);
echo off
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% user defined simulation system
Sim  = arx2arx(Mix);                   % convert estimated model to simulator

Fac2 = facarxls(2, []);
Fac4 = facarxls(4, []);
Facs = { Sim.Facs{1}, Fac2, Sim.Facs{2}, Fac4};
Sim  = mixconst(Facs, 1:4, 1);

Sim.states.Facs{1}.useCth = 1;         % simulation with Cth
Sim.states.Facs{3}.useCth = 1;         % simulation with Cth

% pre-allocated data, references

DATA = zeros(4, maxdat);

% channels
chns = [1 3];                          % list of outputs

% number of repetitions

[res, tstop] = simeval(Sim, chns, maxrep, maxdat, threshold);

y = res{1};
y.stats;

y = res{2};
y.stats;

tra   = res{1}.tra;

%[maxrep, maxdat];
%disp(size(tra)

sub=sub+1; subplot(sub);
plot(tstop, '-o'); grid on;
H=legend('Stationary times');
set(H, 'fontSize', LegendSize); 
drawnow
fprintf('mean of stop time=%g  std=%g\n',mean(tstop),std(tstop));

sub=sub+1; subplot(sub);
[x, ii] = sort(tstop);
i = ii(1);
plot(tra(i, 1:tstop(i)));
hold on
i = ii(length(ii));
plot(tra(i, 1:tstop(i)), ':');
grid on
H=legend('shortest trajectory','longest tajectory');
set(H, 'fontSize', LegendSize); 
plt('zvermimo_')

dial2;

end %===============================================


